package ACL2::Orden::Certifybook;
use base ACL2::Orden;

#
#    Orden CERTIFY-BOOK
#    Copyright (C) 2006-2008 Antonio García Domínguez
#    $Id$
#
#    This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

use strict;
use warnings;
use ACL2::Tags;
use ACL2::Localizacion;

my $lh = ACL2::Localizacion->get_handle;

=head2 get_nombre_libro($this)

Uso: ME<eacute>todo pE<uacute>blico.

Responsabilidades: Devuelve el nombre corto (ruta no completa) del
libro incluido.

Entrada: Ninguna.

Salida: Cadena con el nombre corto del libro incluido en la orden.

=cut

sub get_nombre_libro
{
    return $_[0]->{name};
}

=head2 get_ruta_libro

Uso: ME<eacute>todo pE<uacute>blico.

Responsabilidades: Obtiene la ruta a travE<eacute>s de la cual se ha
de acceder al libro.

Excepciones: El libro no es accesible con la misma
capitalizaciE<oacute>n o en minE<uacute>sculas de la ruta relativa.

Devuelve: Ruta relativa a travE<eacute>s de la cual se debe acceder al
libro.

=cut

sub get_ruta_libro
{
    my ($self, $volumen, $dirBase) = @_;

    # Al ser un certify-book, accedemos al contenido del libro
    my $ruta_completa = File::Spec->catpath($volumen, $dirBase, 
					 $self->get_nombre_libro . ".lisp");
    $ruta_completa = File::Spec->catpath($volumen, $dirBase, 
					 lc $self->get_nombre_libro . ".lisp")
	if ! -f $ruta_completa;

    # No se encontró el libro
    croak $lh->maketext("No se pudo localizar el libro [_1]", 
			$self->get_nombre_libro)
	if ! -f $ruta_completa;
    
    return $ruta_completa;
}

sub protegido_inicializar_sexpr
{
    my ($this) = @_;

    $this->{name} =  $this->get_elems_formula->[1];
    $this->{name} =~ s/"//g;

    $this->{k}           = $this->get_elems_formula->[2] || 0;
    $this->{compileFlag} = $this->get_elems_formula->[3] || "T";
}

sub protegido_inicializar_salida
{
    my ($this) = @_;

    # No guardaremos el nombre presente en la S-expresión original -
    # ACL2 genera el nombre absoluto por su cuenta
    my $nombreCorto      = $this->get_elems_formula->[1];
    my $salida = ${$this->get_salida};

    # Dividimos en pasos
    my ($intro, $calculoChecksum, $prueba, 
	$inclusion, $escrCert, $compilacion) = 
	    split /\s*\* Step [0-9]:\s*/, $salida;

    # Descartamos la salida
    $this->set_salida("");

    # INTRO: Obtención de nombre completo de libro
    ($this->{nombreLibro}) = $intro =~ /CERTIFICATION ATTEMPT FOR "(.*)"/;

    # PASO 1: Nos limitamos a extraer la salida
    $this->{paso1_salida} = $calculoChecksum;

    # PASO 2: Dividimos en órdenes y hacemos las demostraciones
    {
	($this->{paso2_salida}, my @subordenes) = split /(\w*?)\s*>>\s*(.*)$/m, $prueba;

	# Repartimos los elementos debidamente
	my @paquetes = ();
	my @ordenes  = ();
	my @salidas  = ();

	# Ahora vamos añadiendo las subórdenes
	$this->{paso2_ordenes} = [];
	while (@subordenes > 0) {
	    my ($paquete, $sexpr, $salida) = splice @subordenes, 0, 3;
	    push @{$this->{paso2_ordenes}}, ACL2::Orden::Fabrica->crear($sexpr, $salida, $paquete);
	}
    }

    # Paso 3 - ACL2 realiza una orden INCLUDE-BOOK con el fichero probado
    $this->{paso3_orden} = ACL2::Orden::Fabrica->crear("(include-book $nombreCorto)", 
						       $inclusion, $this->get_package);
    
    # Pasos 4 y 5 - nos limitamos a capturar la salida
    $this->{paso4_salida} = $escrCert;
    $this->{paso5_salida} = $compilacion;
}

sub protegido_cabecera
{
    my ($this, $writer) = @_;

    $writer->startTag(ACL2::Tags::CERTIFYBOOK_TAG,
		      "name" => $this->{name},
		      "path" => File::Spec->abs2rel($this->{nombreLibro}),
		      "k"           => $this->{k},
		      "compileFlag" => $this->{compileFlag});
}

sub protegido_cuerpo
{
    my ($this, $writer) = @_;

    # PASO 1
    $writer->startTag(ACL2::Tags::CHECKSUM_TAG);
    ACL2::Orden->salida_parrafos($writer, $this->{paso1_salida});
    $writer->endTag(ACL2::Tags::CHECKSUM_TAG);

    # PASO 2
    $writer->startTag(ACL2::Tags::VERIFICATION_TAG);
    ACL2::Orden->salida_parrafos($writer, $this->{paso2_salida});
    foreach my $orden (@{$this->{paso2_ordenes}}) {
	$orden->escribir_xml($writer);
    }
    $writer->endTag(ACL2::Tags::VERIFICATION_TAG);

    # PASO 3
    $writer->startTag(ACL2::Tags::INCLUSIONCHECK_TAG);
    $this->{paso3_orden}->escribir_xml($writer);
    $writer->endTag(ACL2::Tags::INCLUSIONCHECK_TAG);

    # PASO 4
    $writer->startTag(ACL2::Tags::WRITECERT_TAG);
    ACL2::Orden->salida_parrafos($writer, $this->{paso4_salida});
    $writer->endTag(ACL2::Tags::WRITECERT_TAG);

    # PASO 5
    $writer->startTag(ACL2::Tags::COMPILE_TAG);
    ACL2::Orden->salida_parrafos($writer, $this->{paso5_salida});
    $writer->endTag(ACL2::Tags::COMPILE_TAG);
}

1;
